<table>
    <tbody>
        {% for row in table_data.body.rows %}
            <tr hx-swap-oob="outerHTML:#tr-{{ row.id }}" class="{% if row.is_active %}is-primary{% endif %}" {% if row.hx_get %}hx-get="{{ row.hx_get }}{% querystring detail=row.id %}" hx-swap="none" hx-push-url="{% querystring detail=row.id %}"{% endif %}>
                {% include 'ui/table.html#td' with row=row %}
            </tr>
        {% endfor %}
    </tbody>
    {% if footer %}
        <tfoot>
            <tr hx-swap-oob="innerHTML:#tf-row">
                {% include 'ui/table.html#tf' with table_data=table_data %}
            </tr>
        </tfoot>
    {% endif %}
</table>
